# -*- coding: utf-8 -*-
"""
Created on Tue Jun 12 09:58:27 2018

@author: mmathise
"""


import numpy as np
import ctypes




DLL = ctypes.WinDLL (r'C:\Users\Desktop\Python_DLL\ZMID520X_OneStepCalibration_32.dll')    # Note: When writing the path for the DLL, keep the 'r' at the beginning to avoid Unicode error
filename = 'spa.csv' 
spa_or_analog_input = 0         # Input file: 0 = spatial angle values, 1 = analog values
zmid_type = 1                   # 1=ZMID5201 (analog output), 2=ZMID5202 (PWM), 3=ZMID5203 (SENT)
position_slope = 0              # position_slope defines the slope of the spatial angle values. 0 = positive; 1 = negative
reverse_slope = 0               # reverse_slope: 0 = use same slope as position_slope for Pos0 values; 1 = reverse position_slope
out_mod = 0                     # 0 = Linear output mode; 1 = Modulo360 output mode
clamping_percent_low = 5        # percentage of lower clamping. No clamping = 5
clamping_percent_high = 95      # percentage of higher clamping. No clamping = 95
Vdd_value = 4.994               # supply voltage [V]. Only for input file with analog values. With spatial angle as input file, the Vdd_value will not affect the calculation

### END OF USER INPUT





file_spatial_angle = np.loadtxt(filename,delimiter=',',usecols=range(0,1),skiprows=0)
file_analog_values = np.loadtxt(filename,delimiter=',',usecols=range(0,1),skiprows=0)


DLL.ZMID520X_GetDllVersion.restype = ctypes.c_double
dll_version = DLL.ZMID520X_GetDllVersion()
print('\nDLL version: ' + str(dll_version))


DLL.ZMID520X_PercentToRaw.argtypes = [ctypes.c_double, ctypes.c_int]
DLL.ZMID520X_PercentToRaw.restype = ctypes.c_double

out_min = DLL.ZMID520X_PercentToRaw(clamping_percent_low, zmid_type)
out_max = DLL.ZMID520X_PercentToRaw(clamping_percent_high, zmid_type)


size_spatialangle = len(file_spatial_angle)
size_analog_values = len(file_analog_values)


spatialangle = (ctypes.c_double * (len(file_spatial_angle)))(*file_spatial_angle)
analog_values = (ctypes.c_double * (len(file_analog_values)))(*file_analog_values)

corr = (ctypes.c_double * 9) ()     
corr_reg = (ctypes.c_double * 9) () 

p1 = ctypes.c_double()
p2 = ctypes.c_double()
p3 = ctypes.c_double()
p4 = ctypes.c_double()
p5 = ctypes.c_double()
p6 = ctypes.c_double()
p7 = ctypes.c_double()

clamp_low = ctypes.pointer(p1)      
clamp_high = ctypes.pointer(p2)
clamp_reg = ctypes.pointer(p3)      
offset = ctypes.pointer(p4)         
offset_reg = ctypes.pointer(p5)     
slope = ctypes.pointer(p6)          
slope_reg = ctypes.pointer(p7)      






DLL.ZMID520X_OneStepCalibration.argtypes = [ctypes.c_int32, 
                                            (ctypes.c_double * size_spatialangle), 
                                            ctypes.c_int, 
                                            ctypes.c_double, 
                                            ctypes.c_double, 
                                            ctypes.c_double, 
                                            ctypes.c_double, 
                                            ctypes.c_double, 
                                            ctypes.POINTER(ctypes.c_double), 
                                            ctypes.POINTER(ctypes.c_double), 
                                            ctypes.POINTER(ctypes.c_double), 
                                            ctypes.POINTER(ctypes.c_double),
                                            (ctypes.c_double * 9), 
                                            (ctypes.c_double * 9), 
                                            ctypes.POINTER(ctypes.c_double), 
                                            ctypes.POINTER(ctypes.c_double), 
                                            ctypes.POINTER(ctypes.c_double)]    



DLL.ZMID520X_OneStepCalibration_Analog.argtypes = [(ctypes.c_double * size_analog_values), 
                                                   ctypes.c_int, 
                                                   ctypes.c_double, 
                                                   ctypes.c_double, 
                                                   ctypes.c_double, 
                                                   ctypes.c_double, 
                                                   ctypes.c_double, 
                                                   ctypes.c_double, 
                                                   ctypes.POINTER(ctypes.c_double), 
                                                   ctypes.POINTER(ctypes.c_double), 
                                                   ctypes.POINTER(ctypes.c_double),
                                                   ctypes.POINTER(ctypes.c_double),
                                                   (ctypes.c_double * 9), 
                                                   (ctypes.c_double * 9), 
                                                   ctypes.POINTER(ctypes.c_double), 
                                                   ctypes.POINTER(ctypes.c_double), 
                                                   ctypes.POINTER(ctypes.c_double)]




                           

DLL.ZMID520X_OneStepCalibration.restype = ctypes.c_int32
DLL.ZMID520X_OneStepCalibration_Analog.restype = ctypes.c_int32



if spa_or_analog_input == 1:
    one_step_calibration_analog = DLL.ZMID520X_OneStepCalibration_Analog(analog_values, 
                                                                         size_analog_values, 
                                                                         Vdd_value, 
                                                                         position_slope, 
                                                                         reverse_slope, 
                                                                         out_min, 
                                                                         out_max, 
                                                                         out_mod, 
                                                                         offset, 
                                                                         offset_reg, 
                                                                         slope, 
                                                                         slope_reg, 
                                                                         corr, 
                                                                         corr_reg, 
                                                                         clamp_low, 
                                                                         clamp_high,
                                                                         clamp_reg)

if spa_or_analog_input == 0:
    one_step_calibration_spa = DLL.ZMID520X_OneStepCalibration(zmid_type, 
                                                               spatialangle, 
                                                               size_spatialangle, 
                                                               position_slope, 
                                                               reverse_slope, 
                                                               out_min, 
                                                               out_max, 
                                                               out_mod, 
                                                               offset, 
                                                               offset_reg, 
                                                               slope, 
                                                               slope_reg, 
                                                               corr, 
                                                               corr_reg, 
                                                               clamp_low, 
                                                               clamp_high, 
                                                               clamp_reg)



def corr_reg_to_hex(correction_value):
    corr_to_int = int(correction_value)         
    corr_to_hex = hex(corr_to_int).upper()      
    correction = corr_to_hex[2:]               
    correction = '{:0>2}'.format(correction)   
    return correction


def to_hex(value):
    value_to_int = int(value)
    value_to_hex = hex(value_to_int).upper()
    correction = value_to_hex[2:]
    correction = '{:0>4}'.format(correction)
    return correction
    





def return_code (function):
    if function == 0:
        print('\n')
        print('Write to EEPROM:')
        print('Address 0x00: ' + str(to_hex(offset_reg.contents.value)))
        print('Address 0x01: ' + str(to_hex(slope_reg.contents.value)))
        print('Address 0x02: ' + str(to_hex(clamp_reg.contents.value)))
        print('Address 0x03: ' + str(corr_reg_to_hex(corr_reg[1])) + str(corr_reg_to_hex(corr_reg[0])))
        print('Address 0x04: ' + str(corr_reg_to_hex(corr_reg[3])) + str(corr_reg_to_hex(corr_reg[2])))
        print('Address 0x05: ' + str(corr_reg_to_hex(corr_reg[5])) + str(corr_reg_to_hex(corr_reg[4])))
        print('Address 0x06: ' + str(corr_reg_to_hex(corr_reg[7])) + str(corr_reg_to_hex(corr_reg[6])))
        print('Address 0x07: 00' + str(corr_reg_to_hex(corr_reg[8])))
        print('\nCode 0: SUCCESS. The function returned without errors')
    elif function == -1:
        print('\nCode -1: ERR_INVALID_INPUT. One or more of the inputs are invalid.')
    elif function == -2:
        print('\nCode -2: ERR_SLOPE. The calculated slope is out of range')
    elif function == -3:
        print('\nCode -3: ERR_OFFSET. The calculated offset is out of range')
    elif function == -4:
        print('\nCode -4: ERR_CORR. One or more of the calculated correction values are out of range')
    elif function == -5:
        print('\nCode -5: ERR_POS0. One or more of the calculated Pos0 values are out of range')
    elif function == -6:
        print('\nCode -6: ERR_POS1. One or more of the calculated Pos1 values are out of range')


if spa_or_analog_input == 0:
    return_code(one_step_calibration_spa)
elif spa_or_analog_input == 1:
    return_code(one_step_calibration_analog)
    
    
